/* ts7200io.c
 * Functions for reading/writing to ADC
 *   on TS7200 SBC
 * Jim Jackson
 *
 */
/*
 * Copyright (C) 2004 Jim Jackson           jj@franjam.org.uk
 */
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program - see the file COPYING; if not, write to 
 *  the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
 *  MA 02139, USA.
 */
/*
 *  History......................................................
 */

/***MANPAGE***/
/*
Jim Jackson   <jj@franjam.org.uk>
 */

/***INCLUDES***/
/* includes files 
 */

#include <stdio.h>
/* for SYSV variants .... */
/* #include <string.h> */
#include <strings.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <ctype.h>
#include <sys/time.h>
#include <sys/mman.h>

#include "ts7200io.h"

/***DEFINES***/
/* defines
 */

#define IOMMAP(A)  (mmap(0,getpagesize(),PROT_READ|PROT_WRITE, MAP_SHARED, \
				iofd,A))

#define ADC_LOOP_LIMIT (2048)
/*#define ADC_SLEEP*/
#define TRUE 1
#define FALSE 0

/***GLOBALS***/
/* global variables
 */

int iofd=-1;

int adcerror=0;

/* AD Stuff
 */

volatile unsigned short *adc_io_stat,*adc_io_read;
volatile unsigned char *adc_io_cntl;

/***DECLARATIONS***/
/* declare non-int functions 
 */

/* adc_init   checks that the hardware is there
 *            then sets up some pointers for subsequent use
 *            returns 0 if all ok else returns ENXIO
 */

adc_init()
{
  if (iofd==-1) {
    iofd=open("/dev/mem", O_RDWR | O_SYNC);
    if (iofd==-1) { return(errno); }
  }
  adc_io_cntl=(unsigned char *)IOMMAP(0x22400000);
  if ( ! (*adc_io_cntl & 1) ) { return(ENXIO); }

  adc_io_cntl=(unsigned char *)IOMMAP(0x10c00000);
  adc_io_stat=(unsigned short *)IOMMAP(0x10800000);
  adc_io_read=(unsigned short *)IOMMAP(0x10c00000);

  return(0);
}

/* adc_read(c,r) tell ADC to start a measurent on channel c range r
 *               and wait for it to complete then return answer
 *
 * on error return a value > 65536. Error will be returned value >> 16
 * and also in global variable adcerror
 */

int adc_read(c,r)
int c,r;
{
  int i;
  unsigned char s;
  unsigned short ss;
  short sv;

#ifdef DEBUG 
  if (debug) { fprintf(stderr,"range=%d   chan=%d\n",r,c); fflush(stderr); }
#endif
  s=(r<<3)+c+0x40;
#ifdef DEBUG
  if (debug) { fprintf(stderr,"control word=%02x\n",s); fflush(stderr); }
#endif
  *adc_io_cntl=s;
  for (i=ADC_LOOP_LIMIT; i && ( *adc_io_stat & 0x80 ); i-- ) {
#ifdef DEBUG
    if (debug) { fprintf(stderr,"=%04x\n",*adc_io_stat); fflush(stderr); } 
#endif
#ifdef ADC_SLEEP
    usleep(A);
#endif
  } 
  if (i==0) { return(adcerror=(ETIME<<16)); }
  sv=*adc_io_read;
#ifdef DEBUG
  if (debug) { fprintf(stderr,"adc_io_read=%04x\n",sv); fflush(stderr); }
#endif
  return((int)sv);
}

